/* eslint camelcase: "off" */

const settings = require('./settings')
const BaseAPIClient = require('./BaseAPIClient')

const HEADER_ORG_KEY = 'X-THINKIGO-ORG'
const HEADER_REGION_KEY = 'X-THINKIGO-REGION'
const HEADER_STATION_KEY = 'X-THINKIGO-STATION'

class AdminAPIClient extends BaseAPIClient {
  constructor(baseURI = settings.admin_api.base_uri, apiKey = settings.admin_api.api_key) {
    super(baseURI, { json: true, headers: {} })
    this.apiKey = apiKey
  }

  set orgId(id) {
    this.defaults.headers[HEADER_ORG_KEY] = id
  }

  get orgId() {
    return this.defaults.headers[HEADER_ORG_KEY]
  }

  set regionId(id) {
    this.defaults.headers[HEADER_REGION_KEY] = id
  }

  get regionId() {
    return this.defaults.headers[HEADER_REGION_KEY]
  }

  set stationId(id) {
    this.defaults.headers[HEADER_STATION_KEY] = id
  }

  get stationId() {
    return this.defaults.headers[HEADER_STATION_KEY]
  }

  execute(method, route = '/', options = {}) {
    if (!options.qs) options.qs = {}
    options.qs.api_key = this.apiKey
    const promise = super.execute(method, route, options)
    const catched = promise.catch(resp => Promise.reject(resp.error))
    catched.request = promise.request
    return catched
  }

  checkHealth() {
    return this.get('health-check')
  }

  logIn({ email, password }) {
    return this.post('login', { email, password })
  }

  upsertPackage({ order_id, package_code }) {
    return this.post(`v1/orders/${order_id}/package`, { code: package_code })
  }

  markOrderAsPickedUp({ order_id }) {
    return this.post(`v1/orders/${order_id}/pickup`)
  }

  markOrderAsPacking({ order_id }) {
    return this.post(`v1/orders/${order_id}/pack`)
  }

  markOrderAsReadyForReturn({ order_id }) {
    return this.post(`v1/orders/${order_id}/packed`)
  }

  markOrderAsDelivered({ order_id }) {
    return this.post(`v1/orders/${order_id}/deliver`)
  }

  markOrderAsClosed({ order_id }) {
    return this.post(`v1/orders/${order_id}/close`)
  }

  openPackage({ package_id }) {
    return this.post(`v1/packages/${package_id}/open`)
  }

  unverifiedItems({ test_accounts }) {
    return this.get(`v1/inventory/unverified`, { test_accounts })
  }

  lookupAccounts({ query }) {
    return this.get(`v1/accounts/lookup`, { q: query })
  }

  getAccount({ account_id }) {
    return this.get(`v1/accounts/${account_id}`)
  }

  updateAccount({ account_id, phone }) {
    return this.post(`v1/accounts/${account_id}/`, { phone })
  }

  updateAccountStatus({ account_id, status }) {
    return this.post(`v1/accounts/${account_id}/status`, { status })
  }

  getStorageBill({ account_id }) {
    return this.get(`v1/accounts/${account_id}/storage_bill`)
  }

  createStorageOrder({ account_id }) {
    return this.post(`v1/accounts/${account_id}/create_storage_order`)
  }

  getLoginToken({ account_id }) {
    return this.post(`v1/accounts/token/${account_id}`)
  }

  lookupItem({ item_code }) {
    return this.get(`v1/items/lookup/${item_code}`)
  }

  getOrderManifest({ order_id }) {
    return this.get(`v1/orders/${order_id}/items`)
  }

  markItemAsAddedToBag({ item_code }) {
    return this.get(`v1/items/add-to-bag/${item_code}`)
  }

  getOrderStats({
    type = 'check-ins',
    start,
    end,
    item_details,
    remote,
    special_or_employee,
    special,
    employee,
    test,
    company,
    status,
    order_type
  }) {
    return this.get(`/v1/stats/${type}`, {
      start,
      end,
      item_details,
      remote,
      special_or_employee,
      special,
      employee,
      test,
      company,
      status,
      order_type
    })
  }

  createItem({
    name,
    status,
    stack_id,
    client_uuid,
    account_id,
    package_id,
    location_code,
    weight,
    width,
    height,
    length,
    damaged,
    closed_container,
    large_item,
    concierge_notes,
    rent_audit_status,
    images = [/* { client_uuid, content_type, main_image, file_name, body } */]
  } = {}) {
    const item = {
      name,
      status,
      stack_id,
      client_uuid,
      account_id,
      weight,
      width,
      height,
      length,
      damaged,
      private: closed_container,
      oversized: large_item,
      concierge_notes,
      rent_audit_status
    }

    const qs = package_id ? { package_id, location_code } : { location_code }
    const promise = this.execute('PUT', 'v1/items/', { qs })
    const form = promise.request.form()

    form.append('api_key', this.apiKey)
    form.append('item_data', JSON.stringify(item))
    images.forEach(({ client_image_uuid, content_type, main_image, file_name, body }) => {
      const headers = [
        `--${form.getBoundary()}`,
        `Content-Disposition: form-data; name="${client_image_uuid}"; filename="${file_name}"`,
        `Content-Type: ${content_type}`,
        `Content-Transfer-Encoding: base64`
      ]
      if (main_image) {
        headers.push('Thinkigo-Image-Main: true')
      }
      headers.push('')
      headers.push('')
      form.append(client_image_uuid, body, {
        header: headers.join('\r\n'),
        contentType: content_type
      })
    })

    // NOTE: this'll stall the test, but it's here in case you're curious...
    // form.pipe(process.stdout)
    // -- or --
    // form.pipe(require('fs').createWriteStream('debug.txt'))

    return promise
  }

  adminUpdateItem({
    item_id,
    name,
    status,
    stack_id,
    client_uuid,
    account_id,
    package_id,
    location_code,
    weight,
    width,
    height,
    length,
    damaged,
    closed_container,
    large_item,
    concierge_notes,
    rent_audit_status,
    images = [/* { client_uuid, content_type, main_image, file_name, body } */]
  } = {}) {
    const item = {
      id: item_id,
      name,
      status,
      stack_id,
      client_uuid,
      account_id,
      weight,
      width,
      height,
      length,
      damaged,
      private: closed_container,
      oversized: large_item,
      concierge_notes,
      rent_audit_status
    }

    const qs = package_id ? { package_id, location_code } : { location_code }
    const promise = this.execute('POST', `v1/items/${item_id}`, { qs })
    const form = promise.request.form()

    form.append('api_key', this.apiKey)
    form.append('item_data', JSON.stringify(item))
    images.forEach(({ client_image_uuid, content_type, main_image, file_name, body }) => {
      const headers = [
        `--${form.getBoundary()}`,
        `Content-Disposition: form-data; name="${client_image_uuid}"; filename="${file_name}"`,
        `Content-Type: ${content_type}`,
        `Content-Transfer-Encoding: base64`
      ]
      if (main_image) {
        headers.push('Thinkigo-Image-Main: true')
      }
      headers.push('')
      headers.push('')
      form.append(client_image_uuid, body, {
        header: headers.join('\r\n'),
        contentType: content_type
      })
    })

    // NOTE: this'll stall the test, but it's here in case you're curious...
    // form.pipe(process.stdout)
    // -- or --
    // form.pipe(require('fs').createWriteStream('debug.txt'))

    return promise
  }

  upsertItemImages({
    item_id,
    images = [/* { client_uuid, content_type, main_image, file_name, body } */]
  } = {}) {
    const promise = this.execute('POST', `v1/images/${item_id}`)
    const form = promise.request.form()

    form.append('api_key', this.apiKey)
    images.sort((a, b) => {
      if (a.main_image === b.main_image) {
        return 0
      } else if (a.main_image) {
        return -1
      }
      return 1
    }).forEach(({ client_uuid, file_name, content_type, body }) => {
      const headers = [
        `--${form.getBoundary()}`,
        `Content-Disposition: form-data; name="${client_uuid}"; filename="${file_name}"`,
        `Content-Type: ${content_type}`, '', '']
      form.append(client_uuid, body, { header: headers.join('\r\n'), contentType: content_type })
    })

    // NOTE: this'll stall the test, but it's here in case you're curious...
    // form.pipe(process.stdout)
    // -- or --
    // form.pipe(require('fs').createWriteStream('debug.txt'))

    return promise
  }

  transferInventory({ target_code, location_code }) {
    return this.post('v1/inventory/transfer', { target: target_code, location: location_code })
  }

  updateOrderSchedule({ order_id, datetime }) {
    return this.post(`v1/orders/${order_id}/schedule`, { datetime })
  }

  createOrderNotes({ order_id, notes }) {
    // Call execute directly.
    // Can't use this.put() because it invokes the default form behaviour of the request library,
    // which uses Content-Type: application/x-www-form-urlencoded.
    return this.execute('PUT', `v1/orders/${order_id}/notes`, { body: { notes } })
  }

  getOrderNotes({ order_id }) {
    return this.get(`v1/orders/${order_id}/notes`)
  }

  appendOrderNotes({ order_id, notes }) {
    return this.execute('POST', `v1/orders/${order_id}/notes`, { body: { notes } })
  }

  deleteOrderNotes({ order_id }) {
    return this.delete(`v1/orders/${order_id}/notes`)
  }
}
module.exports = AdminAPIClient
