/* eslint camelcase: "off" */

const settings = require('./settings')
const BaseAPIClient = require('./BaseAPIClient')
const promiseRetry = require('promise-retry')

const AUTH_HEADER_KEY = 'X-THINKIGO-AUTH'

class ConsumerAPIClient extends BaseAPIClient {
  constructor(baseURI = settings.consumer_api.base_uri) {
    super(baseURI, { json: true, headers: { } })
  }

  set credential(credential) {
    this.defaults.headers[AUTH_HEADER_KEY] = credential
  }

  get credential() {
    return this.defaults.headers[AUTH_HEADER_KEY]
  }

  authenticate(method, payload) {
    return this.post(method, payload).then(profile => {
      this.credential = profile.credential
      return profile
    })
  }

  form(method, route, body = {}) {
    return this.execute(method, route, { body })
  }

  execute(method, route, options = {}) {
    return promiseRetry(retry => super.execute(method, route, options).catch(resp => {
      if (resp.statusCode === 504) {
        return retry(resp.error)
      }
      throw resp.error
    }), { retries: 3 })
  }

  getStatus() {
    return this.get('status')
  }

  signUp({ first_name, last_name, password, email, register_phone, signup_code, zip }) {
    return this.authenticate('accounts/register', {
      first_name,
      last_name,
      password,
      email,
      register_phone,
      signup_code,
      zip
    })
  }

  login({ email, password }) {
    return this.authenticate('accounts/login', { email, password })
  }

  loginFacebook({ token }) {
    return this.authenticate('accounts/login-facebook', { token })
  }

  loginAdmin({ token }) {
    return this.post('accounts/login-admin', { token })
  }

  getProfile() {
    return this.get('accounts/me')
  }

  updateProfile({ first_name, last_name, email, username, phone, verification_code, signup_code, apns_token, avatar }) {
    return this.patch('accounts/me', { first_name, last_name, email, username, phone, verification_code, signup_code, apns_token, avatar })
  }

  verifyPhone({ phone }) {
    return this.post('accounts/verify-phone', { phone })
  }

  registerPhone({ registration_code }) {
    return this.post('accounts/register-phone', { registration_code })
  }

  checkUsername({ username }) {
    return this.get('accounts/check-username', { username })
  }

  updatePrimaryAddress({ street, zip, instructions, city, state, latitude, longitude, title, apt }) {
    return this.put('addresses/primary', { street, zip, instructions, city, state, latitude, longitude, title, apt })
  }

  getPrimaryAddress() {
    return this.get('addresses/primary')
  }

  getAddressAvailability({ zip }) {
    return this.get('addresses/availability', { zip })
  }

  updatePrimaryPaymentMethod({ token }) {
    return this.put('payment-methods/primary', { token })
  }

  getPrimaryPaymentMethod() {
    return this.get('payment-methods/primary')
  }

  getBilling() {
    return this.get('billing')
  }

  getBillingSnapshot() {
    return this.get('billing/snapshot')
  }

  getBillingPlan() {
    return this.get('billing/plan')
  }

  resetPassword({ email }) {
    return this.post('accounts/reset-password', { email })
  }

  updatePassword({ email, new_password, verification_code }) {
    return this.post('accounts/update-password', { email, new_password, verification_code })
  }

  getPickupAvailability({ zip, pickup_size, item_ids }) {
    item_ids = item_ids ? JSON.stringify(item_ids) : undefined
    return this.get('pickups/timeslots', { zip, pickup_size, item_ids })
  }

  schedulePickup({ timestamp, pickup_size, notes, item_ids }) {
    return this.post('pickups/schedule', { timestamp, pickup_size, notes, item_ids })
  }

  getCancellationFee({ order_id }) {
    return this.get(`fees/cancellation?id=${order_id}`)
  }

  cancelOrder({ id, reason }) {
    return this.post('orders/cancel', { id, reason })
  }

  getOrderManifest({ order_id }) {
    return this.get('orders/manifest', { id: order_id })
  }

  getOrderTotal({ item_ids, start_day, end_day, timestamp, roundtrip }) {
    return this.get('orders/total', { item_ids, start_day, end_day, timestamp, roundtrip })
  }

  getItem({ id }) {
    console.log('this: ', this)
    console.log('this.get: ', this.get)
    return this.get('items/metadata', { id })
  }

  updateItem({ id, name, category, notes, favorited, availability, is_borrowable, is_rentable, rent_price_day, brand, size }) {
    return this.patch('items/metadata', { id, name, category, notes, favorited, availability, is_borrowable, is_rentable, rent_price_day, brand, size })
  }

  getDeliveryAvailability({ zip, item_ids, stack_ids }) {
    item_ids = item_ids ? JSON.stringify(item_ids) : undefined
    stack_ids = stack_ids ? JSON.stringify(stack_ids) : undefined
    return this.get('deliveries/timeslots', { zip, item_ids, stack_ids })
  }

  scheduleDelivery({ timestamp, roundtrip, item_ids, stack_ids, notes }) {
    return this.post('deliveries/schedule', { timestamp, roundtrip, item_ids, stack_ids, notes })
  }

  authorizeTransfer({ phone, item_ids, stack_ids, name, email, notes }) {
    return this.post('transfers/authorize', { phone, item_ids, stack_ids, name, email, notes })
  }

  acceptTransfer({ authorization_id }) {
    return this.post('transfers/accept', { id: authorization_id })
  }

  cancelTransfer({ authorization_id }) {
    return this.post('transfers/cancel', { id: authorization_id })
  }

  getConnections() {
    return this.get('connections/friends')
  }

  getConnectionRequests() {
    return this.get('connections/requests')
  }

  requestConnection({ ids }) {
    return this.post('connections/request', { ids })
  }

  acceptConnection({ ids }) {
    return this.post('connections/accept', { ids })
  }

  rejectConnection({ ids }) {
    return this.post('connections/reject', { ids })
  }

  createBlockage({ item_id, start_day, end_day }) {
    return this.post(`items/${item_id}/blockages`, { start_day, end_day })
  }

  getBlockage({ item_id }) {
    return this.get(`items/${item_id}/blockages`)
  }

  updateBlockage({ item_id, blockage_id, start_day, end_day }) {
    return this.put(`items/${item_id}/blockages/${blockage_id}/`, { start_day, end_day })
  }

  deleteBlockage({ item_id, blockage_id }) {
    return this.delete(`items/${item_id}/blockages/${blockage_id}/`)
  }

  getNotifications() {
    return this.get('notifications/')
  }

  markNotificationSeen({ ids, unread }) {
    return this.post('notifications/seen', { ids, unread })
  }

  getNotificationCount() {
    return this.get('notifications/count')
  }

  getAppointments() {
    return this.get('appointments')
  }

  searchItems({ query, audience, start_day, end_day, brand, size, min_price, max_price, order, sort, is_rentable }) {
    return this.get('items/search', { query, audience, start_day, end_day, brand, size, min_price, max_price, order, sort, is_rentable })
  }

  searchAccounts({ query }) {
    return this.get('accounts/search', { query })
  }

  getTransactions() {
    return this.get('accounts/transactions')
  }

  getBrands({ query }) {
    return this.get('brands', { query })
  }

  getSizes({ query }) {
    return this.get('sizes', { query })
  }

  lookupCoupon({ code }) {
    return this.get('coupons/lookup', { code })
  }

  applyCoupon({ code }) {
    return this.post('coupons/apply', { code })
  }
}
module.exports = ConsumerAPIClient
